from Acquisition import Implicit
from ZODB.PersistentMapping import PersistentMapping
from types import StringType, ListType

from Products.Gideon.Objects.Packages import PackageVersion
from Products.Gideon.Objects.Packages import PackageFile



import os, urllib, urlparse
from xml.sax import make_parser

import PPDHandler
import CiphonHandler
import PEP241Handler

class MetaAdapter(Implicit):

    """

    an adapter provides a bridge between a specific
    meta data format and the internal format used
    by the repository. this class is an interface

    """

    def __init__(self, name):
        self.name = name
    
    def translate(self, metadata):
        return metadata

"""
we need pass back the info but also other objects
that might be relevant... versions with depends info,
authors
"""

_marker = []

class RepositoryInfo(PersistentMapping):



    def __init__(self):
        PersistentMapping.__init__(self)

    def __getitem__(self, name):
        print 'a',name
        val = self.data.get(name,_marker)
        if val is not _marker: return val
        if name in empty_default: return ''
        
class PPDAdapter(MetaAdapter):

    def process(self, stream):
        
        h = PPDHandler.PPDHandler()
        p = make_parser()
        p.setContentHandler(h)
        p.parse(stream)

        return self.translate(h.get_package_info())
        
    def translate(self, metadata):

        info = RepositoryInfo()

        info['name'] =metadata['name']
        info['summary'] = metadata['title']
        info['description'] = metadata['abstract']

        author = metadata['author']
        print author, type(author)
        if author.count('('):
            idx = author.index('(')
            name = author[:idx]
            email = author[idx:]
            
            info['authors']=[(name, email)]
        else:
            info['authors']=[(name,)]
            
        version = metadata['version']
        implementations = metadata['implementations']

        version = version.replace(',','.')

        v = PackageVersion.PackageVersion(version)

        # what the heck do we do we these ;)
        # i wanna scream, but i have no voice.
        # they are useful but don't directly correspond
        # to a generic process, since we would like
        # to go ahead and retrieve them and add them to the
        # version.. ugh... ok, lets just do it for  now.
        
        # ideally the upload process would fork into
        # some path specific to the current metadata
        # and we could get user input to ok the downloads.

        for i in implementations:

            platform  = i['os']
            arch      = i['arch']
            url       = i['codebase']
            pyversion = i['python_version']

            file = urlparse.urlparse(url)[2].split('/')[-1]
            file_type = os.path.splitext(file)[-1]
            
            #fh = urllib.urlopen(url)
            #import cStringIO
            #fh = cStringIO.StringIO(fh.read())

            f = PackageFile.PackageFile(id=file,
                                        file_type=file_type,
                                        platform = platform,
                                        arch = arch,
                                        py_version = pyversion
                                        )
            #f.manage_upload(fh)

            """
            if i.has_key('dependencies'):
                for d in i['dependencies']:
            """
            
            v.add_file(f)

        return {'info':info, 'version':v}

class CiphonAdapter(MetaAdapter):

    def process(self, stream):

        h = CiphonHandler.CiphonHandler()
        p = make_parser()
        p.setContentHandler(h)
        p.parse(stream)

        return self.translate(h.get_package_info())

    def translate(self, metadata):
        
        from Products.Gideon.Objects.Package import PackageVersion

            
        info = RepositoryInfo()
        
        info['name']    = metadata['name']
        info['license'] = metadata.get('license', None)

        version = metadata.get('version', None)
        
        info['authors'] = metadata.get('authors', [])

        if version:
            
            v = PackageVersion.PackageVersion(version)
            
            dependencies = metadata.get('dependencies', [])

            count = 0
            for d in dependencies:
                count += 1
                di = PackageDependency(
                    str(count),
                    d['name'],
                    'python',
                    d['type'],
                    d['version']
                    )

                v.add_version_dependency(di)

        return {'info':'info', 'version':v}

class PEP241Adapter(MetaAdapter):

    def process(self, stream):

        return self.translate(PEP241Handler.parse_stream(stream))

    def translate(self, metadata):

        info = RepositoryInfo()
        
        name     = metadata['name']
        
        summary  = metadata.get('summary', None)
        license  = metadata.get('license', None)
        platform = metadata.get('platform', None)
        url      = metadata.get('home-page', None)
        keywords = metadata.get('keywords', None)
        version  = metadata.get('version', None)
        author   = metadata.get('author', None)        

        author_email = metadata.get('author-email', None)        
        description  = metadata.get('description', None)

        info['name']=name
        
        if summary and summary != 'UNKNOWN':
            info['summary']=summary

        if description and description != 'UNKNOWN':
            info['description']=description

        if license and license != 'UNKNOWN':
            info['license']=license

        if url and url != 'UNKNOWN':
            info['homepage']=url
            
        if platform:
            
            if isinstance(platform, StringType) and platform != 'UNKNOWN':
                info['platforms']=[platform]
            elif isinstance(platform, ListType):
                info['platforms']=platform
            else: # either unknown or not a string
                pass
            
        if author and author != 'UNKNOWN':
            
            if author_email and author_email != 'UNKNOWN':
                info['authors']=[(author, author_email)]
                
            else:
                info['authors']=[(author,)]
        
        if version and version != 'UNKNOWN':
            v = PackageVersion.PackageVersion(version)
        else:
            v = None

        if keywords and keywords != 'UNKNOWN':
            
            if keywords.count(','):
                keywords = [k.strip() for k in keywords.split(',')]

            info['categories']=keywords
            
        return {'info':info, 'version':v}
    

