
import calendar
from DateTime import DateTime

from Products.Gideon.Objects.Namespace import *
from Products.Gideon.Objects.Catalog import CatalogSchema
import Config

_marker = []

class CalendarEvent(SimpleItem):

    meta_type = 'Calendar Event'
    
    event_widget = DTMLFile('ui/CalendarEventWidget',globals())

    def __init__(self,
                 id,
                 person,
                 resource,
                 task,
                 time_begin,
                 time_end
                 ):
        
        self.id = id
        self._person = person
        self._task = task
        self._resource = resource
        self._time_begin = time_begin
        self._time_end = time_end

    def person(self): 
        return self._person
    def task(self): 
        return self._task
    def resource(self): 
        return self._resource
    def time_begin(self): 
        return self._time_begin
    def time_end(self): 
        return self._time_end
        

calendar_indexes = {
    'person':'FieldIndex',
    'resource':'FieldIndex',
    'task':'FieldIndex',
    'time_begin':'FieldIndex',
    'time_end':'FieldIndex'
    }

calendar_columns = ['person',
                   'resource',
                   'task',
                   'path',
                   'time_begin',
                   'time_end']

calendar_schema = CatalogSchema.CatalogSchema( calendar_indexes, calendar_columns)




def get_week_idx_for_date(date_time):
    montharray = calendar.monthcalendar(date_time.year(), date_time.month())
    day = date_time.day()
    for widx in range(len(montharray)):
        if day in montharray[widx]: return widx
        
    # should never! happen
    raise "Error Retrieving Calendar Week for Date"
    

class CalenderManager(Folder):

    meta_type = 'Calendar Manager'
    
    _default_view_type = 'week'
    
    index_html = None
    
    calendar_nav_widget = DTMLFile('ui/CalendarNavWidget', globals())
                                  
    security = ClassSecurityInfo()

    def __init__(self, id, title):
        self.id = id
        self.title = title

    def __call__(self, REQUEST):
        curtime = DateTime()
        container = self.get_container_for_time(curtime, self._default_view_type)
        return container.index_html(REQUEST)

    security.declarePrivate('add_event')
    def add_event(self, person, resource, task, time_begin, time_end):
        event = CalendarEvent(person, resource, task, time_begin, time_end)
        self.add_event_obj(event)
    
    security.declarePrivate('add_event')
    def add_event_obj(self, event):
        ## events should always get added to the lowest leaf container
        container = self.get_container_for_event(event, 'day')
        container.add_event_obj(event)
        
        ## index event
        wrapped_event = container._getOb(event.getId())
        catalog = self.get_calendar_catalog()
        catalog.catalog_object(wrapped_event)

    security.declarePrivate('has_event')
    def has_event(self, event):
        container = self.get_container_for_event(event, 'day')
        return event.getId() in container.objectIds()
    
    security.declarePrivate('find_event')
    def find_event(self, query): 
        catalog = self.get_calendar_catalog()
        return catalog(**query)

    security.declarePrivate('get_calendar_catalog')
    def get_calendar_catalog(self):
        return self._getOb(Config.CALENDAR_CAT_ID)

    security.declarePublic('get_calendar_manager')
    def get_calendar_manager(self):
        return self

    def get_calendar_paths(self, obj_url):
        path = self.getPhysicalPath()
        path2 = obj_url.split('/')[3:]
        res_paths = []
        for p in path2:
            if p not in path: res_paths.append(p)
        return res_paths
    
    security.declarePublic('get_calendar_trail')
    def get_calendar_trail(self, obj_url):
        """ this should be better """
        res = []
        ob = self
        for p in self.get_calendar_paths(obj_url):
            ob = ob._getOb(p)
            res.append(ob)
        return res    
    
    def get_container_for_time(self, date_time, container_type='week', default=_marker):
        
        if container_type == 'year':
            timeset = (date_time.year(),)
        elif container_type == 'week':
            timeset = (date_time.year(), date_time.month(), get_week_idx_for_date(date_time))
        elif container_type =='day':
            timeset = (date_time.year(), date_time.month(), get_week_idx_for_date(date_time), date_time.day() )
            
        try:
            ob = self
            for tid in timeset:
                ob = ob._getOb(str(tid))
        except Exception, e:
            if default is _marker: raise e
            else: return default
        return ob
    
    def get_container_for_event(self, event, container_type='week', default=_marker):
        return self.get_container_for_time(event.time_begin(), container_type)
        
    def add_calendar_year(self, year):
        
        assert type(year) == type(1), "Years must be integers"
        
        yearobj = CalendarYear(str(year), str(year))
        self._setObject(str(year), yearobj)
        yearobj = self._getOb(str(year))
        
        for m in range(1,13,1):            
            month = CalendarMonth(str(m), calendar.month_name[m])
            yearobj._setObject(str(m), month)
            month = yearobj._getOb(str(m))
            
            montharray = calendar.monthcalendar(year, m)            
            
            for w in range(len(montharray)):
                week = CalendarWeek(str(w), str(w))
                month._setObject(str(w), week)
                week = month._getOb(str(w))
                
                weekarray = montharray[w]
                
                days = filter(None, weekarray)
                    
                for d in days:
                    day = CalendarDay(str(d),str(d))
                    week._setObject(str(d), day)
                    #day._p_deactivate()
                        
                #week._p_deactivate()
                    
            #month._p_deactivate()

        # deactivating doesn't have meaning in this context        
        # would have to do a sub trans commit while holding a ref
        # and then deactivate.
     
    def manage_afterAdd(self, item, container):
        ## event channel stuff
        #channel = self.get_event_channel()
        
        ## add in one year
        curtime = DateTime()
        self.add_calendar_year(curtime.year())
        
        ## add catalog
        CatalogSchema.install_catalog_schema(Config.CALENDAR_CAT_ID, calendar_schema, self)

        ## propogate event
        Folder.manage_afterAdd(self, item, container)
        
InitializeClass(CalenderManager)        
        
class CalendarContainer(Folder):
    meta_type = 'Calendar Container'
    
    def __init__(self, id, title):
        self.id = id
        self.title = title    
    
    def add_event_obj(self, event):
        self._setObject(event.id, event)
        
    def get_container_events(self): 
        # sort by time
        events = self.objectValues('Calendar Event')
        events.sort(lambda x,y: cmp(x.time_begin(), y.time_begin()))
        return events

class CalendarYear(CalendarContainer):        
    
    meta_type = 'Calendar Year'
    
    index_html = DTMLFile('ui/CalendarYearView', globals())

class CalendarMonth(CalendarContainer):
    
    meta_type = 'Calendar Month'
    
    index_html = DTMLFile('ui/CalendarMonthView', globals())
        
class CalendarWeek(CalendarContainer):
    
    meta_type = 'Calendar Week'
    
    index_html = DTMLFile('ui/CalendarWeekView', globals())
    
class CalendarDay(CalendarContainer):

    meta_type = 'Calendar Day'

    index_html = DTMLFile('ui/CalendarDayView', globals())
    
    week_widget = DTMLFile('ui/CalendarDayWeekWidget', globals())
    
    def get_day(self):
        """ """
        year, month, week, day = self.get_calendar_paths(self.absolute_url())
        return calendar.day_name[calendar.weekday(int(year), int(month), int(day))]
        
class PeriodDataSet:
    """
    helper class for displaying timed data sets
    """
    __allow_access_to_unprotected_subobjects = 1
    
    

