from types import StringType, TupleType

from ComputedAttribute import ComputedAttribute
from BTrees.IOBTree import IOSet
from OFS.Image import cookId

from Products.Gideon.Objects.Namespace import *
from Products.Gideon.PageContract.Scripts import compile
from Products.Gideon import FileSystem
from Products.Gideon import MetaDataParsers
from Products.Gideon.Log.Log import log
		
from PackageInfo import PackageInfo
from PackageFile import PackageFile
from Package import Package
from PackageVersion import PackageVersion


class PackageUpload(Folder):

        """
        
        Upload Process Outline
        
        - metadata upload
        - version  upload
        - [edit dependencies]
        - edit categories
        - verify authors/maintainer status
       
        """

        security = ClassSecurityInfo()

        meta_type = "Package Upload"
	
        index_html = DTMLFile('ui/PackageUploadView', globals())

        # start
        metadata_upload_form = DTMLFile('ui/PackageMetadataUploadForm',
                                        globals())


        # steps
        version_upload_form = DTMLFile('ui/PackageVersionUploadForm',
                                       globals())	

        
        # terminals

        upload_finished = DTMLFile('ui/PackageUploadFinished', globals())
	
        package_exists   = DTMLFile('ui/PackageUploadPackageExists',
				globals())
	
	
        def __init__(self,
		 id,
		 title):

                self.id = id
                self.title = title	 

        """
        def authors_submit(self, REQUEST=None):
               pass
                
        def dependencies_submit(self, REQUEST=None):
               pass
        """
        
        def version_upload_submit(self, REQUEST=None):
	        """ """

		contract = """
		#parameters
		package_name:trim
		version_file:file
		file_type:trim, constraint(supported_file_types)
		platform:trim, constraint(supported_platforms)
		pyversion:trim, constraint(major_python_versions)
                new_version_id:optional,trim
                has_dependency: optional, trim, checkbox
		"""
		contract = compile(contract, self)
		contract.validate(REQUEST)

		vars = contract.extract_variables(REQUEST)
		
		queue = self.get_inner_application(self).get_package_queue()

		package = queue.get_package(vars.package_name)

                pvc = package.get_version_container()

                if pvc.get_versions():
                       
                       version = pvc.get_current_version()
                else:
                       version_id = vars.get('new_version_id', None)

                       if version_id:
                              v = PackageVersion(version_id)
                              pvc.add_version(v)
                              
                              version = pvc.get_current_version()
                       else:
                              raise "Invalid Version Specification"
		
		id, title = cookId('','', vars.version_file)

		if not version.has_file(id):

			fobj= PackageFile(id, 
					  title, 
					  vars.file_type,
					  'N/I', # not important right now
					  vars.platform)
			
			version.add_file(fobj)

			fobj = version._getOb(id)

			fobj.file_upload(vars.version_file)
			
		else: # hmm.. good question

                       log.warning ('Overwriting version file')

                       fobj = version.get_file(id)

                       fobj.manage_upload(vars.version_file)

                #################################
                # TODO latter
                #
		# first we check if the info is sufficient
                # currently this logic is added to the Admin for approval
                # logic
                
                #################################
                # we want to check if this person is an author
                # and if not make them a maintainer.
                #
                # ughh. this is a lot more work than it should be...
                #
                
                directory = self.get_application_for_name('Members')
                workspace = directory.get_current_workspace()
		profile = workspace.get_collaborator_for_name('ProfileManager')

                email   = profile.public_email
                name    = profile.name

                info = package.get_package_info()

                found = 0
                for author in info.get_authors():
                       if isinstance(author, TupleType):
                              if name in list(author):
                                     found = 1
                              if email in list(author):
                                     found = 1
                       
                if not found: # not an author
                       maintainers = list(package.get_maintainers())
                       
                       if not (name, email) in maintainers: 
                              maintainers.append( (name, email) )
                              package.set_maintainers(tuple(maintainers))
                       
                #################################
		# send them to a congrats page
                #
		return self.upload_finished(self,
					    REQUEST,
					    package_name=vars.package_name)
		
		if REQUEST is not None:
			REQUEST.RESPONSE.redirect(return_url)


	def metadata_upload_submit(self, REQUEST):
               
		"""
		metadata upload
		version  upload
		[edit dependencies]
		edit categories
		verify authors/maintainer status
		"""

		contract = """
		#parameters
		metadata_file:file
		metadata_format:constraint(supported_metadata_formats)
		"""

		contract = compile(contract, self)
		contract.validate(self)

		vars = contract.extract_variables(REQUEST)
		
		meta  =  MetaDataParsers.process( vars.metadata_format,
						  vars.metadata_file)

		print meta['info']
		
		pi = PackageInfo(info=meta['info'])
		
		# check to make sure we don't already exist
		if self.get_inner_application(self).has_package( 
			pi.package_name()) or \
			self.PackageQueue.has_package(pi.package_name()):

			return self.package_exists(self,
				   REQUEST,
				   package_name=pi.package_name())
		
		# create the package
		p  = Package(pi.package_name())
		p.set_package_info(pi)

		# store the package in the queue
		log.debug('Storing Package in Queue')

		app = self.get_inner_application(self)
		package_queue= app.get_package_queue()
		package_queue.add_package(pi.package_name(), p)		

		# add any extracted versions
		if meta.has_key('version'):
			p.get_version_container().add_version(
				meta['version'])
		
		# lets go ahead and upload versions now

		# PPD is net centric so we offer a separate
		# page to load straight from the uris in the metadata
		if vars.metadata_file == 'PPD':
			
			return self.version_upload_ppd_form(self,
				    REQUEST,
				    package_name=pi.package_name()
				    )

		return self.version_upload_form(self,
				REQUEST,
				package_name=pi.package_name()
				)
	
		
InitializeClass(PackageUpload)
		


	

	
