"""

Task Scheduling, with support for persistent tasks stored
in the zodb, and some consideration of zeo setups.

some of the work on this was lost in a disk failure :(

"""


from time import time

from threading import Thread, Event, RLock
from thread import get_ident

import weakref

from UserList import UserList

from ZopeContext import ZopeContextFactory
import ZEOScheduler

from OFS.SimpleItem import SimpleItem
from OFS.Folder import Folder
from OFS.PropertyManager import PropertyManager

_marker = []

class Scheduler(Thread):

    def __init__(self, *args, **kwargs):
        self._next_time = None
        self._stop = None
        Thread.__init__(self, *args, **kwargs)

    def add_periodic_task(self,
                          task,
                          calendar_unit,
                          hour,
                          minute,
                          startdatetime=None,
                          context=None):

        """ run a task periodically """

        period = TaskTimePeriod(calendar_unit, hour, minute, startdatetime)
        
        self.add_task(task, period)

    def add_delayed_task(self,
                       task,
                       time_ticks, 
                       context=None
                       ):

        """ run once at time_delta from now and remove """
    
        period = TaskTimePeriod(time_ticks=time_ticks)

        self.add_task(task, period)

    def add_task(self, task, period, ignore=0):

        handler  = TaskHandler(task, period)

        task_handler_storage.add_object(task.name, handler)
        
        if ignore: return
        
        self.schedule()        
        
    def get_running(self):

        handlers = task_handler_storage.get_objects()

        return [h for h in  handlers if h.running_p]

    def delete_task(self, name):

        task_handler_storage.remove_object(name)

    def enable_task(self, name):

        handler = task_handler_storage.get(name)

        handler.enable()        
        
    def disable_task(self, name):

        handler = task_handler_storage.get(name)

        handler.disable()

    def delete_all(self):

        map(task_handler_storage.remove_object, task_handler_storage.keys())

    def run(self):
        
        loop = 0

        print 'starting scheduler'
        
        while 1:

            loop += 1
            timeout = None
            schedule_change_event.clear()

            if loop >=10:
                print 'loop break'
                return
            
            current_time = time()

            if self._stop: return

            if self._next_time:
                
                timeout = current_time - self._next_time

            print 'timeout', timeout

            if timeout is None:
                
                schedule_change_event.wait()
                print 's wake up', time()-self._next_time
                continue

            elif timeout > 0:
                
                schedule_change_event.wait(timeout)
                print 's wake up', time()-self._next_time
                continue

            # timeout is negative, someone wants to run
            
            handles = task_handler_storage.objects()

            print 'running tasks'
            
            for h in handles:

                next_time = h.get_next_time()
                
                if next_time is None:
                    # clean up run once activities
                    print 'cleaning up', h.name()
                    self.delete_task(h.name())
                    continue
                
                elif not next_time:
                    # disabled
                    continue

                elif next_time < current_time:
                    # future
                    continue

                print 'running ', h.name()
                h.run()

            self._next_time =  None
            self.schedule()

    def run_task(self, name):
        """ running task reseting its time_delta from this completion """

        handler = task_handler_storage.get(name)
        
        handler.run()

        self.schedule()

    def schedule(self):
        """ find the next time we should run """

        print 'scheduling'

        handles = task_handler_storage.objects()

        time_set = filter(lambda x: x is not None or x > 0,
                          [h.get_next_time() for h in handles]
                          )

        # set _next_time to None?
        if not time_set: return

        next_time = min(time_set)

        print next_time

        if self._next_time and self._next_time <= next_time:
            return
        
        print 'waking up s'
        
        self._next_time = next_time
        schedule_change_event.set() # notify

    def stop(self):
        
        self._stop = 1
        
class TaskHandler:
    
    def __init__(self, task, period):

        self.task = task
        self.period = period
        
        self.history = TaskHistory()

        self.running_p = 0
        
        self.start_time = None
        self.finish_time = None
        self.register_time = time()

        # i hate circular refs
        self.period.handler = weakref.ref(self)

    def run(self):

        self.start_time = time()
        
        self.thread = Thread(None, self.task._run, self.task.name, (self,)) 

        self.thread.start()
        
        self.running_p = 1

    def _task_completed(self):

        self.finish_time = t = time()
        
        self.history.append(t)

        self.running_p = 0        

    def enable(self):

        self.period.startdatetime = DateTime()

    def disable(self):
        
        self.period.startdatetime = None

    def get_next_time(self):

        return self.period.next()

    def name(self):
        return self.task.name

class TaskTimePeriod:

    supported_calendar_units = ('Month', 'Week', 'Day')

    def __init__(self,
                 calendar_unit=None,
                 hour=None,
                 minute=None,
                 startdatetime=None,
                 time_ticks=None):

        if calendar_unit is None and time_ticks is None:
            raise SyntaxError ("Must Provide a unit or time ticks ")

        self.unit = calendar_unit
        self.hour = hour or 0
        self.minute = minute or 0
        self.startdatetime = startdatetime
        self.time_ticks = time_ticks

    def time_delta(self):

        # time offset
        offset = (self.hour*3600+self.minute*60)/(24*60*60)

        startdate = self.startdatetime
        
        if self.unit == 'Month':
            return (startdate.earliestTime()+offset+30).timeTicks()
        elif self.unit == 'Week':
            return (startdate.earliestTime()+offset+7).timeTicks()
        elif self.unit == 'Day':
            return (startdate.earliestTime()+offset)



    def next(self):
        """
        get the next timeticks that this task wants to be run at

        if we recieve a time ticks than we need to return
        it if we haven't already run. if we have then its done
        and we return None, if we're disabled we return 0.

        sigh. lots of implicit values here.
        """
        
        if self.time_ticks is not None:
            
            if self.handler().start_time is None:
                return self.time_ticks
            else:
                return None

        else:
            
            if self.period.startdatetime is None:
                return 0
            else:
                return self.time_delta()
            
class SubClassResponsiblity(Exception): pass
        
class Task:

    """ should be subclassed"""

    def __init__(self, name):
        
        self.name = name

    def run(self):
        
        raise SubClassResponsiblity("Subclass should implement run method")

    def _run(self, handler):

        self.run()
        
        handler._task_completed()        
        
    
class FunctionTask(Task):

    """ procedural alternative """

    def set_function(self, function):
        
        self.function = function

    def run(self):
        
        self.function()

class ZopeTask(Task): 

    """

    calls a persistent callable object 

    lots of caveats
    
    """

    def _run(self):
        
        from Testing import makerequest
        
        request = makerequest(zope_context())

        obj = request.unrestrictedTraverse(self.task.name)
        
        # we should zlog exceptions
        obj()

        self.handler._task_completed()
            
class TaskHistory(UserList): pass

### Persistent Stuff Task


class PersistentTask(SimpleItem, PropertyManager):

    _properties = (

        {'id':'title',
         'type':'string',
         'mode':'w'},
        
        {'id':'calendar_unit',
         'type':'selection',
         'mode':'w',
         'selection':'supported_calendar_units'},

        {'id':'hour',
         'type':'string',
         'mode':'w'},

        {'id':'minute',
         'type':'string',
         'mode':'w'},

        {'id':'startdate',
         'type':'date',
         'mode':'w'
         },
        
        {'id':'target',
         'type':'string',
         'mode':'w'}
        )

    manage_options = (
        {'label':'Configure', 'action':'manage_propertiesForm',
         'help':('OFSP','Properties.stx')},
        ) + SimpleItem.manage_options
        
    def __init__(self, id, title):

        self.id = id
        self.title = title

    def make_transient(self):
        """ returns a transient handler with a transient task"""

        task = ZopeTask(self.make_task_name(), task)

    def make_task_name(self):
        return '/'.join(getattr(self, target).getPhysicalPath())
    
    def supported_calendar_units(self):
        return TaskTimePeriod.supported_calendar_units

    def manage_afterAdd(self, item, container):
        pass


    def manage_beforeDelete(self, *args):

        scheduler.delete_task(self.make_task_name())


class PersistentTaskContainer(Folder):

    def __setstate__(self, state):
        
        PersistentProxy(self)

        self.__dict__.update(state)

class PersistentProxy:
    
    def __init__(self, container):
        
        self._p_jar = PersistentTaskContainerRegistration(container)
        
        get_transaction().register(self)
        
class PersistentTaskContainerRegistration:

    def __init__(self, container):

        self.container = container

    def tpc_vote(self, *args):
        
        tasks = self.container.get_tasks()

        for t in tasks:
            
            task_handler_storage.add_object( *(t.make_transient()+(1,)) )
            
        

class ThreadedContainer:

    def __init__(self):

        self._objects = {}
        self._lock    = RLock()

    def add_object(self, key, object, ignore=0):

        self._lock.acquire()
        
        try:
            if not self._objects.has_key(key):
                self._objects[key]=object
            elif ignore:
                pass
            else:
                raise AttributeError("Duplicate Task %s"%key)
        finally:
            self._lock.release()

    def remove_object(self, key): 

        self._lock.acquire()

        if self._objects.has_key(key):
            del self._objects[key]
        
        self._lock.release()
    
    def get_object(self, key, default=_marker):

        self._lock.acquire()

        obj = self._objects.get(key, default)

        if obj is _marker:
            raise AttributeError("Object %s not Found"%str(key))
        
        self._lock.release()

        return obj

    def objects(self):
        
        # careful: someone could delete it after
        # we return the ref
        
        self._lock.acquire()

        objects = self._objects.values()

        self._lock.release()

        return objects
    
    def keys(self):

        keys = self._objects.keys()

        self._lock.release()

        return keys        

    
task_handler_storage = ThreadedContainer()

zope_context = ZopeContextFactory()

schedule_change_event = Event()

scheduler = Scheduler(name='Gideon Scheduler')

scheduler.setDaemon(1)

if ZEOScheduler.check():
    scheduler.start()



