"""
 (C) Copyright 2002 Kapil Thangavelu <kvthan@wm.edu>
 All Rights Reserved

 This file is part of Gideon.

 Gideon is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Gideon is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Gideon; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

# $Id: SubscriptionService.py,v 1.2 2002/04/09 10:17:09 ender Exp $

from Products.Gideon.Objects.Namespace import *

from Products.Gideon.Objects.Application.ApplicationObject \
     import ApplicationObject

from Products.Gideon.PageContract.Scripts import compile

from ISubscribable import ISubscriptionProvider
from Subscription import Subscription

class SubscriptionError(Exception): pass

class SubscriptionDelegate(SimpleItem):
    """
    This class provides a useful interaction between users and 
    objects they wish to subscribe to.
    """

    security = ClassSecurityInfo()

    index_html = None

    subscription_registration_form =DTMLFile('ui/SubscriptionRegistrationForm',
                                             globals())


    def __init__(self, id, title=''):
        self.id = id
        self.title = title

    security.declareProtected('Subscribe to Events', '__call__')
    def __call__(self, REQUEST):
        """ """
        obj = REQUEST.PARENTS.pop(0)

        subservice = self.get_service("SubscriptionService")

        subtypes = subservice.get_subscription_types_for_object(obj)

        if subtypes:
            return self.subscription_registration_form(self,
                             REQUEST,
                             subtypes=subtypes,
                             time_periods=subservice.subscription_time_periods,
                             location='/'.join(obj.getPhysicalPath()),
                             return_url=obj.absolute_url()                    
                             )                          
        else:
            raise SubscriptionError("Not Allowed")

InitializeClass(SubscriptionDelegate)

class SubscriptionService(ApplicationObject, Folder):

    """
    provides system wide management of subscriptions
    """

    security = ClassSecurityInfo()
    
    meta_type = 'Subscription Service'

    subscription_time_periods = ('Daily', 'Weekly', 'Monthly')
    
    def __init__(self, id, title=''):
        
        self.id = id
        self.title = title

    security.declarePrivate('get_subscription_manager_for_user')
    def get_subscription_manager_for_user(self, user_id=None):
        
        if user_id is None:
            user_id = getSecurityManager.getUser().getId()
            
        userdirectory = self.get_application_for_name('Members')
        workspace = userdirectory.get_user_workspace(user_id)

        return workspace.get_collaborator_for_name('SubscriptionsManager')
        
    security.declarePrivate('get_subscription_types_for_object')
    def get_subscription_types_for_object(self, obj):
        
        """
        first we ask the object then we ask the objects
        inner application.
        """

        if ISubscriptionProvider.isImplementedBy(obj):
            return obj.get_subscription_types()
        else:
            app = obj.get_inner_application(obj)
            if ISubscriptionProvider.isImplementedBy(app):
                return app.get_subscription_types_for_subobject(obj)

    security.declarePublic('is_subscription_allowed_for')
    def is_subscription_allowed_for(self, obj):

        if self.get_subscription_types_for_object(obj):
            return 1

    def register_subscription(self, REQUEST):
        """

        """
        
        contract = """
        #parameters
        subscription_id: trim
        event_type: trim
        period: trim
        location: trim
        return_url:trim
        """

        contract = compile(contract)

        v = contract.extract_variables(REQUEST)
        
        user_id = getSecurityManager().getUser().getId()
        
        submanager = self.get_subscription_manager_for_user(user_id)
        
        subscription = Subscription(
                          v.subscription_id,
                          v.event_type,
                          v.location,
                          v.period
                          )

        submanager.add_subscription(v.subscription_id,  subscription)        
        
        # hmm.. how bout an option to go to their subscription manager
        REQUEST.RESPONSE.redirect(v.return_url)
        
    def manage_afterAdd(self, item, container):
        
        from SubscriptionQueue import SubscribableEventQueue
        queue = SubscribableEventQueue('queue')
        self._setObject('queue', queue)
    
        
InitializeClass(SubscriptionService)
